@@ -4,6 +4,8 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/assert"
+
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 )
@@ -15,7 +17,7 @@ func TestHostAvailable(t *testing.T) {
 			Namespace: "myns",
 		},
 	}
-	hostWithError.SetErrorMessage("oops something went wrong")
+	hostWithError.SetErrorMessage(RegistrationError, "oops something went wrong")
 
 	testCases := []struct {
 		Host        BareMetalHost
@@ -318,7 +320,7 @@ func TestHostNeedsDeprovisioning(t *testing.T) {
 					Online: true,
 				},
 			},
-			Expected: false,
+			Expected: true,
 		},
 
 		{
@@ -332,7 +334,7 @@ func TestHostNeedsDeprovisioning(t *testing.T) {
 					Online: true,
 				},
 			},
-			Expected: false,
+			Expected: true,
 		},
 
 		{
@@ -376,24 +378,6 @@ func TestHostNeedsDeprovisioning(t *testing.T) {
 			Expected: false,
 		},
 
-		{
-			Scenario: "externally provisioned",
-			Host: BareMetalHost{
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      "myhost",
-					Namespace: "myns",
-				},
-				Spec: BareMetalHostSpec{
-					ExternallyProvisioned: true,
-					Image: &Image{
-						URL: "same",
-					},
-					Online: true,
-				},
-			},
-			Expected: false,
-		},
-
 		{
 			Scenario: "removed image",
 			Host: BareMetalHost{
@@ -443,10 +427,10 @@ func TestHostNeedsDeprovisioning(t *testing.T) {
 		t.Run(tc.Scenario, func(t *testing.T) {
 			actual := tc.Host.NeedsDeprovisioning()
 			if tc.Expected && !actual {
-				t.Error("expected to need provisioning")
+				t.Error("expected to need deprovisioning")
 			}
 			if !tc.Expected && actual {
-				t.Error("did not expect to need provisioning")
+				t.Error("did not expect to need deprovisioning")
 			}
 		})
 	}
@@ -547,3 +531,177 @@ func TestCredentialStatusMatch(t *testing.T) {
 		})
 	}
 }
+
+func TestGetImageChecksum(t *testing.T) {
+	for _, tc := range []struct {
+		Scenario string
+		Host     BareMetalHost
+		Expected bool
+	}{
+		{
+			Scenario: "both checksum value and type specified",
+			Host: BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost",
+					Namespace: "myns",
+				},
+				Spec: BareMetalHostSpec{
+					Image: &Image{
+						Checksum:     "md5hash",
+						ChecksumType: MD5,
+					},
+				},
+			},
+			Expected: true,
+		},
+		{
+			Scenario: "checksum value specified but not type",
+			Host: BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost",
+					Namespace: "myns",
+				},
+				Spec: BareMetalHostSpec{
+					Image: &Image{
+						Checksum: "md5hash",
+					},
+				},
+			},
+			Expected: true,
+		},
+		{
+			Scenario: "sha256 checksum value and type specified",
+			Host: BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost",
+					Namespace: "myns",
+				},
+				Spec: BareMetalHostSpec{
+					Image: &Image{
+						Checksum:     "sha256hash",
+						ChecksumType: SHA256,
+					},
+				},
+			},
+			Expected: true,
+		},
+		{
+			Scenario: "sha512 checksum value and type specified",
+			Host: BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost",
+					Namespace: "myns",
+				},
+				Spec: BareMetalHostSpec{
+					Image: &Image{
+						Checksum:     "sha512hash",
+						ChecksumType: SHA512,
+					},
+				},
+			},
+			Expected: true,
+		},
+		{
+			Scenario: "checksum value not specified",
+			Host: BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost",
+					Namespace: "myns",
+				},
+				Spec: BareMetalHostSpec{
+					Image: &Image{
+						ChecksumType: SHA512,
+					},
+				},
+			},
+			Expected: false,
+		},
+		{
+			Scenario: "neither checksum value nor hash specified",
+			Host: BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost",
+					Namespace: "myns",
+				},
+				Spec: BareMetalHostSpec{
+					Image: &Image{
+						URL: "someurl",
+					},
+				},
+			},
+			Expected: false,
+		},
+		{
+			Scenario: "wrong checksum hash specified",
+			Host: BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost",
+					Namespace: "myns",
+				},
+				Spec: BareMetalHostSpec{
+					Image: &Image{
+						Checksum:     "somehash",
+						ChecksumType: "boondoggle",
+					},
+				},
+			},
+			Expected: false,
+		},
+		{
+			Scenario: "no image",
+			Host: BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost",
+					Namespace: "myns",
+				},
+				Spec: BareMetalHostSpec{},
+			},
+			Expected: false,
+		},
+	} {
+		t.Run(tc.Scenario, func(t *testing.T) {
+			_, _, actual := tc.Host.GetImageChecksum()
+			if actual != tc.Expected {
+				t.Errorf("expected %v but got %v", tc.Expected, actual)
+			}
+		})
+	}
+}
+
+func TestBootMode(t *testing.T) {
+	for _, tc := range []struct {
+		Scenario  string
+		HostValue BootMode
+		Expected  BootMode
+	}{
+		{
+			Scenario:  "default",
+			HostValue: "",
+			Expected:  UEFI,
+		},
+		{
+			Scenario:  "UEFI",
+			HostValue: UEFI,
+			Expected:  UEFI,
+		},
+		{
+			Scenario:  "legcy",
+			HostValue: Legacy,
+			Expected:  Legacy,
+		},
+	} {
+		t.Run(tc.Scenario, func(t *testing.T) {
+			host := &BareMetalHost{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "myhost",
+					Namespace: "myns",
+				},
+				Spec: BareMetalHostSpec{
+					BootMode: tc.HostValue,
+				},
+			}
+			actual := host.BootMode()
+			assert.Equal(t, tc.Expected, actual)
+		})
+	}
+}
